/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.java.net
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.actor;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import org.joda.time.DateTime;
import org.joda.time.Duration;

/***********************************************************************************************************************
 * 
 * Represents a single task that is possibly decomposed in multiple subtasks and provides support for waiting for it
 * completion.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface Collaboration 
  {
    /*******************************************************************************************************************
     * 
     * A provider of a {@link Collaboration}.
     * 
     ******************************************************************************************************************/
    public static interface Provider 
      {
        @Nonnull
        public Collaboration getCollaboration();
      }

    /*******************************************************************************************************************
     * 
     * Returns the message that originated this {@code Collaboration}.
     * 
     * @return  the message
     * 
     ******************************************************************************************************************/
    @Nonnull
    public Object getOriginatingMessage();
    
    /*******************************************************************************************************************
     * 
     * Returns {@code true} if the {@code Collaboration} has been completed.
     * 
     ******************************************************************************************************************/
    public boolean isCompleted();
    
    /*******************************************************************************************************************
     * 
     * Waits for the completion of this {@code Collaboration}.
     * 
     ******************************************************************************************************************/
    public void waitForCompletion() 
      throws InterruptedException;
    
    /*******************************************************************************************************************
     * 
     * Return the time when this {@code Collaboration} has been created.
     * 
     ******************************************************************************************************************/
    @Nonnull
    public DateTime getStartTime();
    
    /*******************************************************************************************************************
     * 
     * Return the duration of this {@code Collaboration}.
     * 
     ******************************************************************************************************************/
    @Nonnull
    public Duration getDuration();
    
    /*******************************************************************************************************************
     * 
     * Returns the number of messages related to this {@code Collaboration} not yet delivered.
     * 
     ******************************************************************************************************************/
    @Nonnegative
    public int getDeliveringMessagesCount();

    /*******************************************************************************************************************
     * 
     * Returns the number of messages related to this {@code Collaboration} not yet consumed.
     * 
     ******************************************************************************************************************/
    @Nonnegative
    public int getPendingMessagesCount();

    /*******************************************************************************************************************
     * 
     * Returns the number of running threads assigned to this {@code Collaboration}.
     * 
     ******************************************************************************************************************/
    @Nonnegative
    public int getRunningThreadsCount();
  }
